import pygame
from pygame.locals import *

from pykitten import utils
from pykitten.engine import State

from widgets import TextEntry
from resources import JB

class BaseState(State):
    def __init__(self):
        State.__init__(self)
        self.screen = pygame.display.get_surface()
        self.background = pygame.Surface(self.screen.get_size())
        self.background.fill((0,0,0))


class FadeState(BaseState):
    def __init__(self, old_surf=None, new_surf=None, fade=10):
        BaseState.__init__(self)

        if not old_surf:
            old_surf = pygame.Surface((800,600))
            old_surf.fill((0,0,0))
        if not new_surf:
            new_surf = pygame.Surface((800,600))
            new_surf.fill((0,0,0))


        self.background.blit(old_surf, (0,0))
        self.new_surf = new_surf
        self.new_surf.set_alpha(0)

        self.alpha = 0
        self.fade = fade
        self.fps = 30

    def update_other(self):
        #fade surfaces
        self.alpha += self.fade
        self.new_surf.set_alpha(self.alpha)
        if self.alpha >= 180:
            self.quit()

    def update_screen(self):
        self.screen.blit(self.new_surf, (0,0))
        pygame.display.flip()

class SuperSpecialFadeState(FadeState):
    def __init__(self, old_surf, new_surf, fade=10, play=0):
        self.play = play #play menu music on quit
        FadeState.__init__(self, old_surf, new_surf, fade)

    def quit(self):
        if self.play:
            JB.play("bgm.xm")
        FadeState.quit(self)

    def update_other(self):
        FadeState.update_other(self)
        if self.alpha > 50:
            self.fade = 10

class MsgState(BaseState):
    #uses passed surface as bg and paints an empty "window"
    #on center of screen
    def __init__(self, surface, winsize=(400,300)):
        BaseState.__init__(self)
        self.background = surface

        self.scr_update = 1
        self.entry = None

        self.window = pygame.Surface(winsize)
        self.window.fill((10,10,10))

        self.winpos = utils.center(self.window.get_size(), \
            self.screen.get_size())

        #quit by pressing ESCAPE
        self.add_kbevent(KEYDOWN, K_ESCAPE, callback=self.quit)


    def add_textentry(self, pos, max=14):
        self.entry = TextEntry(max)
        self.entry.set_pos(pos)
        self.entry.draw(self.window)

        #K_a = 97, K_z = 122
        keys = [K_a, K_b, K_c, K_d, K_e, K_f, K_g, K_h, K_i, K_j,
        K_k, K_l, K_m, K_n, K_o, K_p, K_q, K_r, K_s, K_t, K_u, K_v,
        K_w, K_x, K_y, K_z,
        K_0, K_1, K_2, K_3, K_4, K_5, K_6, K_7, K_8, K_9, ]

        for i in keys: #all valid keys
            self.add_kbevent(KEYDOWN, i, \
                callback=self.update_textentry, keynr=i)

        #backspace
        self.add_kbevent(KEYDOWN, K_BACKSPACE, \
            callback=self.update_textentry, keynr=None)

        #return key
        self.add_kbevent(KEYDOWN, K_RETURN,
            callback=self.submit_entry)

    def submit_entry(self):
        pass

    def update_textentry(self, keynr):
        self.scr_update = 1
        self.entry.add_character(keynr)

    def update_screen(self):
        if self.scr_update:
            self.scr_update = 0
            if self.entry:
                self.entry.draw(self.window)
            self.screen.blit(self.background, (0,0))
            self.screen.blit(self.window, self.winpos)
            for w in self.widgets:
                w.draw(self.screen)
            pygame.display.flip()
