import os
import pygame
from pygame.locals import *

from pykitten import data
from pykitten import utils
from pykitten.engine import Engine
from pykitten.gui import ImgButton, ClickArea, TextLine

from game import EditorGame
from basestate import BaseState, MsgState
from widgets import TextEntry
from map import MapData, draw_map, draw_pipes, save_map
from resources import *

class DragArea(ClickArea):
    def on_click(self):
        ClickArea.on_release(self)

    def on_release(self):
        pass

class Editor(BaseState):
    def __init__(self, mapdata=None):
        BaseState.__init__(self)
        self.selected = None
        self.map_pos = (0,32)
        self.scr_update = 1

        self.next_was = None

        #tile buttons
        buttons = [
            "delete.png",
            "start.png",
            "finish.png",
            "ground.png",
            "btn.png",
            "del_pipe.png",
            "pipe.png",
            "pipe_exit.png",
            "pipe_turn.png",
            "pipe_tjunc.png",
            "block.png",
        ]

        x, y = 0, 0
        self.tile_buttons = {}
        for b in buttons:
            self.background.blit(GFX_tiles[b], (x,y))
            tile_button = ImgButton(img=GFX_tiles[b],
                callback=self.select_type,
                name=b)
            tile_button.set_pos((x,y))
            self.widgets.append(tile_button)
            self.tile_buttons[b] = tile_button
            x += 34

        self.select_type("delete.png")


        #color buttons
        y = 0
        x = 800 - ((len(COLORS)+1) * 34)

        img = GFX_tiles["del_col.png"] #nocolor button
        self.background.blit(img, (x,y))
        col_button = ImgButton(img,
            callback=self.select_type,
            name="del_col.png")
        col_button.set_pos((x,y))
        self.widgets.append(col_button)
        self.tile_buttons["del_col.png"] = col_button
        x += 34

        img = pygame.Surface((32,32))
        for c in COLORS:  #draw button for each color
            img.fill(c)
            name = COLORS.index(c)
            self.background.blit(img, (x,y))
            col_button = ImgButton(img=img,
                callback=self.select_type,
                name=name)
            col_button.set_pos((x,y))
            self.widgets.append(col_button)
            self.tile_buttons[name] = col_button
            x += 34


        #save, load and test button
        x, y = 704, 584
        buttons = ["save.png", "load.png", "test.png"]
        for b in buttons:
            img = GFX_tiles[b]
            self.background.blit(img, (x,y))
            btn = ImgButton(img,
                callback=self.submit,
                action=b)
            btn.set_pos((x,y))
            self.widgets.append(btn)
            x += 34

        #quit button
        x = 0
        img = GFX_tiles["quit.png"]
        self.background.blit(img, (x,y))
        btn = ImgButton(img, callback=self.quit)
        btn.set_pos((x,y))
        self.widgets.append(btn)

        #new button
        x += 34
        img = GFX_tiles["new.png"]
        self.background.blit(img, (x,y))
        btn = ImgButton(img, callback=self.new_map)
        btn.set_pos((x,y))
        self.widgets.append(btn)


        #map
        if mapdata:
            self.mapdata = mapdata
        else:
            self.mapdata = MapData()
        self.mapcopy = self.mapdata.copy()
        self.map_layer = pygame.Surface((800,544))
        self.map_layer.set_colorkey((255,0,255))
        self.map_layer.fill((20,20,20))
        self.pipe_layer = pygame.Surface((800,544))
        self.pipe_layer.set_colorkey((255,0,255))
        self.pipe_layer.fill((255,0,255))

        draw_map(self.mapdata, self.map_layer)
        draw_pipes(self.mapdata, self.pipe_layer)

        #tile buttons
        for k in self.mapdata.tiles.keys():
            tile = self.mapdata.tiles[k]
            rect = [[tile.pos[0]*32+self.map_pos[0],
                tile.pos[1]*32+self.map_pos[1]], [32,32]]
            btn = DragArea(rect, callback=self.put_tile, coord=k)
            self.widgets.append(btn)

        #grid
        self.grid_layer = pygame.Surface((800,544))
        self.grid_layer.fill((255,0,255))
        self.grid_layer.set_colorkey((255,0,255))
        for i in xrange(1,25):
            pygame.draw.line(self.grid_layer, (255,255,255),(i*32,0), (i*32,544))

        for i in xrange(1,17):
            pygame.draw.line(self.grid_layer, (255,255,255), (0,i*32), (800,i*32))

        #keyboard shortcuts
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)
        #self.add_kbevent(KEYDOWN, K_F3, self.new_map)
        self.add_kbevent(KEYDOWN, K_F5, self.submit, action="test.png")
        self.add_kbevent(KEYDOWN, K_F7, self.submit, action="save.png")
        self.add_kbevent(KEYDOWN, K_F8, self.submit, action="load.png")



    def submit(self, action):
        if action=="save.png":
            self.next = SaveMap(self.screen, self.mapdata)
        elif action=="load.png":
            self.next = LoadMap(self.screen)
        elif action=="test.png":
            #save map to tmp
            self.next_was = "game"
            self.mapcopy = self.mapdata.copy()
            save_map(self.mapdata, "TMP", MAP_folder)
            self.next = EditorGame(self.mapdata)
        else:
            return

        self.pause()
        self.quit()

    def new_map(self):
        self.next = Editor()
        self.quit()

    def unpause(self):
        self.scr_update = 1
        if self.next_was == "game":
            self.next_was = None
            self.mapdata = self.mapcopy.copy()
            #elf.mapdata = load_mapdata("TMP", MAP_folder)
        draw_map(self.mapdata, self.map_layer)
        draw_pipes(self.mapdata, self.pipe_layer)
        BaseState.unpause(self)

    def put_tile(self, coord):
        pipe = self.mapdata.pipes[coord]
        tile = self.mapdata.tiles[coord]
        if self.selected in xrange(0, len(COLORS)): #colorize
            if pipe.type != "empty.png":
                pipe.switch_color(self.selected)
            elif tile.type == "btn.png":
                tile.switch_color(self.selected)
        elif self.selected == "del_col.png": #decolorize
            pipe.switch_color(None)

        elif self.selected=="delete.png":  #delete
            tile.switch_type("empty.png")
        elif self.selected=="del_pipe.png":
            pipe.switch_type("empty.png")

        elif self.selected.startswith("pipe"): #put/rotate pipes
            if pipe.type == self.selected:
                #change rotation on 2nd click
                pipe.add_rot()
            else:
                pipe.switch_type(self.selected)

        elif self.selected=="block.png":  #block junction
            if pipe.type=="pipe_tjunc.png":
                pipe.switch_block()
        else:
            tile.switch_type(self.selected) #put normal tile

        draw_map(self.mapdata, self.map_layer)
        draw_pipes(self.mapdata, self.pipe_layer)
        self.scr_update = 1


    def select_type(self, name):
        def gen_pointlist((x,y)):
            offset = 0
            pointlist = [
                (x-offset, y-offset),
                (x-offset, y+32+offset),
                (x+32+offset, y+32+offset),
                (x+32+offset, y-offset),
            ]
            return pointlist


        self.scr_update = 1

        if self.selected:
            #deselect previous type
            button = self.tile_buttons[self.selected]
            pygame.draw.lines(self.background,
                (0,0,0),
                True,
                gen_pointlist(button.get_pos()))

        #select new type
        self.selected = name
        button = self.tile_buttons[name]
        pygame.draw.lines(self.background,
            (255,255,255),
            True,
            gen_pointlist(button.get_pos()))

    def update_screen(self):
        if self.scr_update:
            self.scr_update = 0
            self.screen.blit(self.background, (0,0))
            self.screen.blit(self.map_layer, (0,32))
            self.screen.blit(self.pipe_layer, (0,32))
            self.screen.blit(self.grid_layer, (0,32))

            pygame.display.flip()




class SaveMap(MsgState):
    def __init__(self, bgsurf, mapdata):
        MsgState.__init__(self, bgsurf)
        wx, wy = self.winpos

        txt = TextLine("Save")
        x = utils.center(txt.get_size(), self.window.get_size())[0]
        y = 10
        self.window.blit(txt, (x, y))

        x = 20
        y += 30
        txt = TextLine("Enter filename:", fontsize=14)
        self.window.blit(txt, (x, y))

        y += 20
        self.add_textentry((x,y))
        if mapdata.filename is not None:
            self.entry.text = mapdata.filename
            self.entry.update()

        x = self.entry.get_size()[0] + 20
        txt = TextLine(".map", fontsize=14)
        self.window.blit(txt, (x,y))

        self.mapdata = mapdata

    def submit_entry(self):
        #save map

        filename = self.entry.text + ".map"
        if save_map(self.mapdata, filename, MAP_folder):
            self.show_msg("Map saved.")
        else:
            self.show_msg("ERROR: Saving map failed.")

        self.quit()

    def show_msg(self, msg):
        #TODO: show message in editor window instead of console
        print msg



class LoadMap(MsgState):
    def __init__(self, bgsurf):
        MsgState.__init__(self, bgsurf)
        wx, wy = self.winpos

        txt = TextLine("Load")
        x = utils.center(txt.get_size(), self.window.get_size())[0]
        y = 10
        self.window.blit(txt, (x, y))

        #get map files
        self.filenames = get_mapnames(MAP_folder)

        self.cur_page = 0
        self.max_lines = 10

        #file buttons
        x = 45
        y = 40
        img = pygame.Surface((300, 14))
        img.fill((0,0,150))
        self.file_buttons = []
        for i in xrange(self.max_lines):
            btn = ImgButton(img,
                callback=self.load_map,
                btn_nr=i)
            btn.set_pos((x+self.winpos[0], y+self.winpos[1]))
            self.file_buttons.append(btn)
            self.widgets.append(btn)
            y += 20

        #scroll buttons
        s = 25
        img = pygame.Surface((s,s)) #draw rectangle
        img.fill((10,10,10))
        pygame.draw.polygon(img, (255,255,255),[(0,0),(s,s/2),(0,s)],1)

        y = y + self.winpos[1] + 20
        x = self.winpos[0] + 220
        btn = ImgButton(img, callback=self.scroll, dir=1)
        btn.set_pos((x,y))
        self.widgets.append(btn)

        x = x - s - 20
        img = pygame.transform.rotate(img, 180)
        btn = ImgButton(img, callback=self.scroll, dir=-1)
        btn.set_pos((x,y))
        self.widgets.append(btn)

        self.draw_filenames()


    def scroll(self, dir):
        if dir < 0 and self.cur_page - 1 < 0:
            return
        elif dir > 0 and \
            ((self.cur_page+1) * self.max_lines) >= len(self.filenames):
            return

        self.cur_page += dir
        self.draw_filenames()


    def draw_filenames(self):
        self.scr_update = 1
        min = self.cur_page * self.max_lines

        for i in xrange(self.max_lines):
            btn = self.file_buttons[i]
            btn.fill((0,0,150))
            nr = i + min
            if nr < len(self.filenames):
                txt = TextLine(self.filenames[nr], fontsize=14,
                    bgcolor=(0,0,150))
                btn.blit(txt, (0,0))


    def load_map(self, btn_nr):
        if btn_nr >= len(self.filenames):
            print "eek!"
            return

        filename = self.filenames[btn_nr + (self.cur_page*self.max_lines)]
        path = data.filepath(MAP_folder, filename)
        try:
            mapdata = data.load_object(path)
            self.show_msg("Map loaded.")
        except:
            self.show_msg("ERROR: Loading map failed.")

        if mapdata:
            self.kill_prev = True
            self.next = Editor(mapdata)
            self.quit()


    def show_msg(self, msg):
        #TODO: show message in editor window instead of console
        print msg



def main():
    #pygame.display.set_mode((800,600))
    e = Engine()
    e.current_state = Editor()
    e.run()
