import pygame

from pykitten.kittens import Kitten

from resources import GFX_wobbly

class Ground(pygame.rect.Rect):
    pass


class GfxMap:
    #has spritegroups with ground, handles collisions and positioning
    def __init__(self, mapdata):

        self.mapdata = mapdata
        self.start = (0,0)
        self.finish = (0,0)

        offset = (0, 23) #ground offset
        size = (32, 9) #real ground size
        self.ground = []
        self.btns = {}
        self.pipe_exits = {}

        for key in mapdata.tiles.keys():  #get map data
            tile = mapdata.tiles[key]
            pipe = mapdata.pipes[key]
            #pipe = mapdata.pipes[key]

            if tile.type=="start.png":
                self.start = tile.pos
            elif tile.type=="finish.png":
                self.finish = get_tilepos(key, center=True)
            elif tile.type == "btn.png":
                x, y = get_tilepos(key, center=True)
                self.btns[key] = (x,y)
            elif pipe.type == "pipe_exit.png":
                x, y = get_tilepos(key, center=True)
                self.pipe_exits[key] = (x,y)



            #get ground
            if tile.type == "ground.png" or \
                tile.type == "start.png" or \
                tile.type == "btn.png" or \
                tile.type == "finish.png":
                x, y = get_tilepos(key)
                y = y + offset[1] #add y_offset of ground
                rect = pygame.rect.Rect([(x,y), size])
                self.ground.append(rect)

    def place_player(self, player, pos=None):
        #put player on start pos
        if pos is None:
            x, y = self.start
        else:
            x, y = pos
        x *= 32
        y = y*32 + 35
        player.set_pos((x,y))


    def check_collisions(self, player):
        if player.mode != 0:
            return

        if player.rect.collidelist(self.ground) > -1:
            if player.falling > 4:
                player.die_fall()

            #move player up
            x, y = player.pos
            y -= player.fall_speed
            player.set_pos((x,y))


    def player_on_btn(self, player):
        #return button color if player near button
        for key in self.btns.keys():
            b = self.btns[key]
            if player.rect.collidepoint(b):
              btn = self.mapdata.tiles[key]
              return btn.color
        return None

    def player_on_pipeexit(self, player):
        for key in self.pipe_exits.keys():
            p = self.pipe_exits[key]
            if player.rect.collidepoint(p):
                return key
        return None

    def player_on_finish(self, player):
        return player.rect.collidepoint(self.finish)

    def use_btn(self, colnr):
        #flip every pipe with given color
        for key in self.mapdata.pipes.keys():
            p = self.mapdata.pipes[key]
            if p.color == colnr:
                if p.type == "pipe_tjunc.png":
                    p.switch_block()
                elif p.type == "pipe.png" or p.type == "pipe_turn.png":
                    p.add_rot()

    def pipe_player(self, player):
        #move player through pipe

        if player.prev_coord:
            next_coord, dead_end=self.next_pipe(player.coord, player.prev_coord)
        else:
            next_coord, dead_end = player.coord, None

        if next_coord is None and dead_end is None:
            dead_end = player.coord

        if dead_end:
            self.place_player(player, dead_end)
            from_p = self.mapdata.pipes[player.coord]
            if dead_end in self.mapdata.pipes.keys() and \
                self.mapdata.pipes[dead_end].type!="pipe_tjunc.png":
                player.unpipe()
                self.place_player(player, dead_end)
            else:
                player.die_pipe()
            return

        cx, cy = next_coord
        pipe = self.mapdata.pipes[cx, cy]

        player.prev_coord = player.coord
        player.coord = next_coord

        x, y = get_tilepos(player.coord)

        if pipe.type=="pipe_tjunc.png":
            #straight: (0,2), (1,3)
            rfree = without_block(pipe)
            if (0 in rfree and 2 in rfree) or \
                (1 in rfree and 3 in rfree):
                img = GFX_wobbly["pipe.png"][0].copy()
            else:
                img = GFX_wobbly["pipe_turn.png"][0].copy()
        else:
            img = GFX_wobbly[pipe.type][0].copy()
        if pipe.rot:
            img = pygame.transform.rotate(img, pipe.rot*90)

        player.image = img
        player.set_pos((x,y))



    def next_pipe(self, coord, prev_coord):
        #find next valid coord on pipe path
        # 0, 1, 2, 3 = up, left, down, right

        x, y = coord
        next = [(x,y-1), (x-1,y), (x,y+1), (x+1,y)]
        pipe = self.mapdata.pipes[coord]
        p_free = without_block(pipe)

        next_coord = None
        dead_end = None
        for f in p_free:
            check = next[f]
            if check == prev_coord:
                continue
            if check in self.mapdata.pipes.keys():
                n_pipe = self.mapdata.pipes[check]
                n_free = without_block(n_pipe)
                if (f==0 and 2 in n_free) or \
                    (f==1 and 3 in n_free) or \
                    (f==2 and 0 in n_free) or \
                    (f==3 and 1 in n_free):
                    next_coord = next[f]
                else:
                    dead_end = next[f]

        return next_coord, dead_end


def without_block(pipe):
            #get free tile without block
            p_free = pipe.free[:]
            if pipe.type=="pipe_tjunc.png":
                b = pipe.block + pipe.rot
                if b > 3:
                    b -= 4
                if b in p_free:
                    p_free.remove(b)
            return p_free

def get_tilepos(pos, center=False):
    x, y = (pos[0]*32, pos[1]*32 + 32)
    if center:
        x += 16
        y += 16 #get center point of tile
    return (x,y)
