import pygame

from pykitten.kittens import AnimatedKitten

from resources import SFX

class MrWobbly(AnimatedKitten):
    #a wobbly kitten? O.o

    def __init__(self, images):
        #images is a dict with "stand" "move" and "pipe" images
        self.img_dict = images
        AnimatedKitten.__init__(self, self.img_dict["stand"], delay=15)
        self.play()

        self.fall_speed = 2
        self.mode = 0 #0 walk mode, 1 pipe mode, 2 pipe death, 3 fall death
        self.coord = None #pos in pipe mode
        self.prev_coord = None
        self.falling = 0

        self.set_vel((0, self.fall_speed))

        r = self.rect
        self.move_rect = pygame.rect.Rect([(0,0), (51, 19)])
        self.stand_rect = self.rect

    def get_pos(self):
        return self.rect.topleft

    def move(self, dir):
        if self.mode != 0: #only move in walk mode
            return
        #dir=-1 to move left, dir=1 to move right, dir=0 to stop
        pos = self.rect.topleft

        if dir:
            self.rect = self.move_rect

            if dir > 0:
                self.set_images(self.img_dict["move"])
                self.rect.topleft = pos
            else:
                self.set_images(self.img_dict["move_l"])
                self.rect.topright = pos
        else:
            self.rect = self.stand_rect
            self.set_images(self.img_dict["stand"])
        self.set_vel((dir, self.vel[1]))

    def fall(self):
        self.falling += 1
        if self.pos[1] > 550: #die when falling on bottom of level
            self.die_fall()


    def unfall(self):
        self.falling = 0

    def die_pipe(self):
        self.mode = 2
        self.set_images(self.img_dict["death"][2:])
        SFX["splash.ogg"].play()

    def die_fall(self):
        self.mode = 2
        self.set_images(self.img_dict["death"])
        SFX["splash.ogg"].play()

    def update(self):
        if self.mode==0:  #walk mode
            AnimatedKitten.update(self)
        elif self.mode==1:
            pass
        elif self.mode==2:
            self.vel = (0,0)
            AnimatedKitten.update(self)
            if self.cur_image == len(self.images) -1:
                self.kill()
                self.mode = 42

    def pipe(self, coord):
        self.mode = 1
        self.coord = coord

    def unpipe(self):
        self.mode = 0
        self.coord = None
        self.prev_coord = None
        self.set_images(self.img_dict["stand"])
        self.move(0)

    def set_images(self, images):
        w1, h1 = self.image.get_size()
        AnimatedKitten.set_images(self, images)
        w2, h2 = self.image.get_size()

        diff_w = (w1 - w2) / 2
        diff_h = (h1 - h2) / 2
        self.set_pos((self.pos[0] + diff_w, self.pos[1] + diff_h))





