import pygame
from pygame.locals import *

from pykitten import utils
from pykitten.gui import ImgButton, ClickArea



from resources import *
from basestate import BaseState
from editor import Editor, LoadMap
from game import CredGame
from campaign import Campaign
from widgets import WobbleButton
from mapselect import MapSelect, ContMapSelect
from intro import Intro

class StartMenu(BaseState):
    def __init__(self):
        if not pygame.mixer.music.get_busy():
            JB.play("bgm.xm", -1)

        BaseState.__init__(self)

        self.background.blit(GFX_bg["mainscreen.png"], (0,0))
        self.fps = 30

        buttons = ["startgame", "continue", "custom", "editor", "exit"]
        #place menu buttons

        w, h = 127, 50
        full_h = 5 * (h + 2)
        x, y = utils.center((w, full_h), (800,600))
        self.wobbles = []
        for b in buttons:
            img1 = GFX_menu[b + "1.png"]
            img2 = GFX_menu[b + "2.png"]
            btn = WobbleButton(img1, img2,
                callback=self.submit, action=b)
            btn.set_pos((x,y))
            self.wobbles.append(btn)
            y = y + h + 2


        r = pygame.rect.Rect([(226,51),(35,30)])
        r = ClickArea(r, callback=self.next_state)
        self.widgets.append(r)

        #keyboard shortcuts
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)

    def next_state(self):
        self.next = CredGame()
        self.pause()
        self.quit()

    def submit(self, action):
        if action=="startgame":
            self.next = Intro()
            self.pause()
            self.quit()
        elif action=="continue":
            self.next = ContMapSelect()
            self.pause()
            self.quit()
        elif action=="custom":
            self.next = MapSelect("custom_maps")
            self.pause()
            self.quit()
        elif action=="editor":
            self.next = Editor()
            self.pause()
            self.quit()
        elif action=="exit":
            self.quit()


    def update_screen(self):
        for w in self.wobbles:
            w.update()
            w.draw(self.background)
        self.screen.blit(self.background, (0,0))
        pygame.display.flip()
