import pygame
import string

from pykitten import utils
from pykitten.gui import StaticWidget, TextLine, ImgButton

class WobbleButton(ImgButton):
    def __init__(self, img1, img2, callback, **kwargs):
        img = pygame.Surface(img1.get_size())
        ImgButton.__init__(self, img, callback, **kwargs)
        self.anim_timer = utils.Timer(20)
        self.images = [img1, img2]
        self.blit(img1, (0,0))
        self.cur_img = 0
        self.anim_timer.counter = self.anim_timer.interval + 1

    def update(self):
        ImgButton.update(self)
        if self.hover and self.anim_timer.update():
            self.cur_img += 1
            if self.cur_img >= len(self.images):
                self.cur_img = 0
            self.blit(self.images[self.cur_img], (0,0))

    def on_unhover(self):
        self.blit(self.images[0], (0,0))
        self.anim_timer.counter = self.anim_timer.interval + 1



class TextEntry(StaticWidget):
    def __init__(self, max=15):
        size = (max * 14, 20)
        StaticWidget.__init__(self, size)
        self.fill((0,0,0))

        self.text_line = TextLine(" " * max, 14)
        self.max = max
        self.text = ""

    def add_character(self, keynr):
        if keynr is None:
            self.rem_character()
            return

        keyname = pygame.key.name(keynr)

        if (keyname in string.ascii_letters or \
            keyname in string.digits) and \
            len(self.text) < self.max:
            self.text = self.text + keyname
            self.update()


    def rem_character(self):
        if self.text:
            self.text = self.text[:-1]
            self.update()

    def update(self):
        self.fill((0,0,0))
        self.text_line.update(self.text)
        self.blit(self.text_line, (0,0))



class TextButton(ImgButton):
    def __init__(self, img, callback, **kwargs):
        ImgButton.__init__(self, img, callback, **kwargs)
        self.bg_image = img.copy()
        self.text = ""
        self.hi_col = (81,255,168)
        self.lo_col = (0,0,0)
        self.bg_col = self.lo_col
        self.fontcolor = (255,255,255)

    def set_text(self, text):
        self.fill(self.bg_col)
        txt = TextLine(text, fontsize=14, bgcolor=self.bg_col,
            color=self.fontcolor)
        self.text = text
        pos = utils.center(txt.get_size(), self.get_size())
        self.blit(txt, pos)

    def highlight(self):
        self.bg_col = self.hi_col
        self.fontcolor = (0,0,0)
        self.set_text(self.text)


    def unhighlight(self):
        self.bg_col = self.lo_col
        self.fontcolor = (255,255,255)
        self.set_text(self.text)



class FlipButton(ImgButton):
    def __init__(self, img1, img2, callback, **kwargs):
        ImgButton.__init__(self, img1, callback, **kwargs)
        self.set_colorkey((255,0,255))
        self.fill((255,0,255))
        self.img1 = img1
        self.img2 = img2


    def on_hover(self):
        self.blit(self.img2, (0,0))

    def on_unhover(self):
        self.fill((255,0,255))


